"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InvalidOrder = void 0;
const errors_js_1 = require("../errors.js");
class InvalidOrder extends errors_js_1.AssertionError {
    constructor(state, method, args, options) {
        super(method === 'end' ? getEndMessage(state) : getExpectingMessage(state, method, args), options);
        Object.defineProperty(this, "state", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: state
        });
        Object.defineProperty(this, "method", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: method
        });
        Object.defineProperty(this, "args", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: args
        });
    }
}
exports.InvalidOrder = InvalidOrder;
function getEndMessage(state) {
    return `Planned for ${state.maxStep} step but expecting step ${state.step} when 'end()' is called`;
}
function getExpectingMessage(state, method, args) {
    return `Expecting ${getExpectingCalls(state)}, but received '${method}(${getMethodArgs(args)})'`;
}
function getExpectingCalls(state) {
    let message;
    if (state.subStep === 0) {
        const methods = ['is', 'once', 'any'];
        message = methods
            .map(m => `'${m}(${['any'].indexOf(m) === -1 ? state.step : `[${state.step}]`})'`)
            .join(', ');
    }
    else {
        const methods = ['exactly'];
        message = methods.map(m => `'${m}(${state.step}, ${state.maxSubStep})'`).join(', ');
    }
    return message;
}
function getMethodArgs(args) {
    return args.map(a => (Array.isArray(a) ? `[${a.join(', ')}]` : a)).join(', ');
}
//# sourceMappingURL=InvalidOrder.js.map