"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.repeat = void 0;
const is_promise_1 = __importDefault(require("is-promise"));
const errors_js_1 = require("../errors.js");
/**
 * Repeat the specified function n times and return the last result.
 * If the result is a promise, it will run the function sequentially.
 */
function repeat(fn, times) {
    return repeatRecur(fn, 1, times);
}
exports.repeat = repeat;
function repeatRecur(fn, i, times) {
    try {
        const result = fn();
        if (i === times)
            return result;
        if ((0, is_promise_1.default)(result)) {
            return result.then(() => repeatRecur(fn, i + 1, times), e => {
                throw new errors_js_1.FailOnOccurrence(i, e, { ssf: repeat });
            });
        }
        else {
            return repeatRecur(fn, i + 1, times);
        }
    }
    catch (e) {
        if (e instanceof errors_js_1.FailOnOccurrence)
            throw e;
        throw new errors_js_1.FailOnOccurrence(i, e, { ssf: repeat });
    }
}
//# sourceMappingURL=repeat.js.map