"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.satisfies = void 0;
const satisfier_1 = require("satisfier");
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../utils/index.js");
/**
 * Check if `actual` satisfies criteria in `expected`.
 * @param expected All properties can be a value which will be compared to the same property in `actual`,
 * RegExp, or a predicate function that will be used to check against the property.
 */
function satisfies(actual, expected) {
    const diff = (0, satisfier_1.createSatisfier)(expected).exec(actual);
    if (diff) {
        throw new errors_js_1.AssertionError((0, index_js_1.notSatisfiedMessage)(diff), { ssf: satisfies });
    }
}
exports.satisfies = satisfies;
//# sourceMappingURL=satisfies.js.map