"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.throws = void 0;
const is_promise_1 = __importDefault(require("is-promise"));
const errors_js_1 = require("../errors.js");
const index_js_1 = require("../utils/index.js");
function throws(value, validator) {
    if (typeof value !== 'function' && !(0, is_promise_1.default)(value)) {
        throw new errors_js_1.AssertionError('`assertron.throws()` must be called with a function or promise.', {
            ssf: throws
        });
    }
    if (!(0, is_promise_1.default)(value)) {
        const { threw, err, result } = tryCatch(value);
        if (threw) {
            if (failValidation(validator, err)) {
                throw new errors_js_1.AssertionError((0, index_js_1.unexpectedErrorMessage)(err, validator), { ssf: throws });
            }
            return err;
        }
        value = result;
    }
    if ((0, is_promise_1.default)(value)) {
        return value.then(value => {
            throw new errors_js_1.AssertionError((0, index_js_1.notRejectedMessage)(value), { ssf: throws });
        }, err => {
            if (failValidation(validator, err)) {
                throw new errors_js_1.AssertionError((0, index_js_1.unexpectedErrorMessage)(err, validator), { ssf: throws, cause: err });
            }
            return err;
        });
    }
    throw new errors_js_1.AssertionError((0, index_js_1.notThrownMessage)(value), { ssf: throws });
}
exports.throws = throws;
function tryCatch(fn) {
    let result;
    let err;
    let threw = false;
    try {
        result = fn();
    }
    catch (e) {
        err = e;
        threw = true;
    }
    return { threw, err, result };
}
function failValidation(validator, error) {
    if (!validator)
        return false;
    if ((0, index_js_1.isErrorConstructor)(validator)) {
        return !(error instanceof validator);
    }
    else {
        return !validator(error);
    }
}
//# sourceMappingURL=throws.js.map