import { createSatisfier } from 'satisfier';
import { AssertionError } from '../errors.js';
import { notSatisfiedMessage } from '../utils/index.js';
/**
 * Check if `actual` satisfies criteria in `expected`.
 * @param expected All properties can be a value which will be compared to the same property in `actual`,
 * RegExp, or a predicate function that will be used to check against the property.
 */
export function satisfies(actual, expected) {
    const diff = createSatisfier(expected).exec(actual);
    if (diff) {
        throw new AssertionError(notSatisfiedMessage(diff), { ssf: satisfies });
    }
}
//# sourceMappingURL=satisfies.js.map