import isPromise from 'is-promise';
import { AssertionError } from '../errors.js';
import { isErrorConstructor, notRejectedMessage, notThrownMessage, unexpectedErrorMessage } from '../utils/index.js';
export function throws(value, validator) {
    if (typeof value !== 'function' && !isPromise(value)) {
        throw new AssertionError('`assertron.throws()` must be called with a function or promise.', {
            ssf: throws
        });
    }
    if (!isPromise(value)) {
        const { threw, err, result } = tryCatch(value);
        if (threw) {
            if (failValidation(validator, err)) {
                throw new AssertionError(unexpectedErrorMessage(err, validator), { ssf: throws });
            }
            return err;
        }
        value = result;
    }
    if (isPromise(value)) {
        return value.then(value => {
            throw new AssertionError(notRejectedMessage(value), { ssf: throws });
        }, err => {
            if (failValidation(validator, err)) {
                throw new AssertionError(unexpectedErrorMessage(err, validator), { ssf: throws, cause: err });
            }
            return err;
        });
    }
    throw new AssertionError(notThrownMessage(value), { ssf: throws });
}
function tryCatch(fn) {
    let result;
    let err;
    let threw = false;
    try {
        result = fn();
    }
    catch (e) {
        err = e;
        threw = true;
    }
    return { threw, err, result };
}
function failValidation(validator, error) {
    if (!validator)
        return false;
    if (isErrorConstructor(validator)) {
        return !(error instanceof validator);
    }
    else {
        return !validator(error);
    }
}
//# sourceMappingURL=throws.js.map